/* $Id: //guest/paul_krause/perforce/integrations/jdev/src/com/perforce/jdev/P4Client.java#8 $
 * Copyright 2000-2002 SoundBite Corporation.   All Rights Reserved.
 */
package com.perforce.jdev;

import com.perforce.api.Client;
import com.perforce.api.FileEntry;

import com.perforce.jdev.cm.P4ConnectionFactory;
import com.perforce.jdev.cm.P4ConnectionType;

import java.io.File;
import java.io.IOException;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import oracle.ide.Ide;

import oracle.ide.controls.WaitCursor;

import oracle.ide.exception.ChangeVetoException;

import oracle.ide.scm.SCMCheckoutLister;
import oracle.ide.scm.SCMClient;
import oracle.ide.scm.SCMEnvironment;
import oracle.ide.scm.SCMFile;
import oracle.ide.scm.SCMFileStatus;
import oracle.ide.scm.SCMFileSystem;
import oracle.ide.scm.SCMOperationSet;
import oracle.ide.scm.SCMPropertyCustomizer;
import oracle.ide.scm.SCMRegistry;
import oracle.ide.scm.SCMSystem;

import oracle.ide.scm.env.SCMUtilitySet;

import oracle.ide.scm.error.SCMException;

import oracle.ide.scm.util.SCMAssert;
import oracle.ide.scm.util.SCMLogWriter;
import oracle.ide.scm.util.SCMVersionFileStatus;

import oracle.jdeveloper.cm.CMException;
import oracle.jdeveloper.cm.ConnectionManager;

import oracle.jdeveloper.model.JProjectFilter;


/**
 * A client adapter for p4's Client.
 *
 * <h4>Contract</h4>
 *
 * SCMClients must do the following:<ul>
 *
 * <li> Register with the source control addin framework using SCMRegistry.
 * <li> Provide an SCMCheckoutList, if you want to use the List Checkouts
 *      window.
 * <li> Return the SCMOperationSet for the categorized set of operations
 *      you want your SCMClient to use.
 * <li> Define the SCMPropertyCustomizer for the client.
 *      This allows the source control addin user to edit source control
 *      client preferences.
 * <li> Return the current version system status of an SCMFile.</ul>
 *
 * @author Paul Krause
 * @created Wed Apr 24 14:07:46 2002
 * @version Release 0 $Revision: #8 $
 * @see Client
 */
public class P4Client implements SCMClient {

   /** This is release 0 */
   private static final int RELEASE = 0;

   /** RCS Revision keyword */
   private static final String REVISION = "$Revision: #8 $";

   /** The p4 log */
   private static SCMLogWriter scmLog;

   /** The commands implemented by this AddIn */
   private SCMOperationSet scmOperations;

   /** The P4Client FileSystem. */
   private P4FileSystem p4FileSystem = new P4FileSystem();

   /** Customizable environment properties */
   private P4PropertySheet p4Props;

   /** Maps file names to SCMFileStates */
   private final Map fileStates = new HashMap(20011);

   /** Element Category of P4 Source nodes */
   public static final int P4_SOURCE = JProjectFilter.newCategory();

   /**
    * Gets and inits the p4 log
    *
    * @return The p4 Log
    */
   public static SCMLogWriter getLog() {
      if (scmLog != null) return scmLog;
      final SCMEnvironment env = SCMSystem.getEnvironment();
      final SCMUtilitySet util = env.getUtils();
      scmLog = util.createLogger("p4");
      return scmLog;
   }

   /**
    * Writes a message to the p4 log window.
    *
    * @param msg The message to log
    */
   public static void log(String msg) {
      getLog().writeln(msg);
   }

   /**
    * Prints a stack trace to p4 log window.
    *
    * @param x The object or class that caught <code>th</code>
    * @param m The name of the method that caught <code>th</code>
    * @param msg Additional explanation or context
    * @param th The caught Throwable
    */
   public static void logException(Object x, String m, String msg, Throwable th)
   {
      final SCMLogWriter log = getLog();
      Class c = x.getClass();
      if (c.equals(Class.class)) c = (Class) x;
      log.writeln(c.getName() + "." + m);
      log.writeln("\t" + msg);
      log.writeln("\t" + x);
      th.printStackTrace(log.getPrintWriter());
   }

   /**
    * The name of this source-code management system.
    *
    * @return Perforce
    */
   public String getSCMName() {
      return "Perforce";
   }

   /**
    * A short version of SCMName, suitable for labels.
    *
    * @return p4
    */
   public String getSCMShortName() {
      return "p4";
   }

   /**
    * Gets the operations supprted by this Addin
    *
    * @return The Operation Set
    * @exception SCMException to signal failure to SCM system
    */
   public SCMOperationSet getOperations() throws SCMException {
      if (scmOperations != null) return scmOperations;
      scmOperations = new SCMOperationSet();
      final Class[] single = new Class[] {
         P4Edit1Cmd.class,
         P4DiffCmd.class,
      };
      final Class[] multi = new Class[] {
         P4AddCmd.class,
         P4EditCmd.class,
      };
      final Class[] toggle = new Class[] {
         P4ToggleCommand.class,
      };
      scmOperations.addCategory(single);
      scmOperations.addCategory(multi);
      scmOperations.addCategory(toggle);
      return scmOperations;
   }

   /**
    * Gets a virtual filesystem for this client's depot.
    *
    * @return The depot FileSystem
    */
   public SCMFileSystem getFileSystem() {
      return p4FileSystem;
   }

   /**
    * Gets the value of the checkout lister property
    *
    * @return The CheckoutLister value
    */
   public SCMCheckoutLister getCheckoutLister() {
      return null;
   }

   /**
    * Gets the value of the property customizer property
    *
    * @return The PropertyCustomizer value
    */
   public SCMPropertyCustomizer getPropertyCustomizer() {
      return null;
   }

   /**
    * Inquires as to the status of a file.
    *
    * @param f The file to check
    * @return The file's SCMStatus
    * @exception SCMException to report failure to SCM System
    */
   public SCMFileStatus getStatus(SCMFile f) throws SCMException {
      final String path = f.getPath();
      SCMFileStatus fs = (SCMFileStatus) fileStates.get(path);
      if (fs != null) return fs;
      if (f.isDirectory() || path.endsWith("~") || f.getFileName().startsWith("#")) {
         return SCMVersionFileStatus.STATUS_UNRECOGNIZED;
      }
      final Client c = p4FileSystem.guessClient(f);
      if (c == null) return SCMVersionFileStatus.STATUS_UNRECOGNIZED;
      if (c.getSyncTime() == 0) {
         initFileStates(c);
         c.inSync();
         c.refreshUpdateTime();
         fs = (SCMFileStatus) fileStates.get(path);
         if (fs != null) return fs;
      }
      fs = P4Node.create(f, c.getEnv());
      fileStates.put(path, fs);
      return fs;
   }

   /**
    * Connects to the depot specified by the client's environment, fetches all
    * {@link FileEntry}s visible to the client's root view, and stores them
    * in {@link #fileStates}, indexed by URL.
    *
    * @arg c The Client to init
    * @exception SCMException if the client root is not in canoncial form
    * @exception IOException if an error occurs
    */
   private synchronized void initFileStates(Client c) throws SCMException {
      getLog().write(c.getName() + " ... ");
      WaitCursor wc = Ide.getWaitCursor();
      try {
         wc.show();
         final Vector fents = FileEntry.getFiles(c.getEnv(), getViewString(c));
         final Iterator fili = fents.iterator();
         while (fili.hasNext()) {
            final FileEntry fe = (FileEntry) fili.next();
            fe.refreshUpdateTime();
            if (fe.getHeadRev() == fe.getHaveRev()) fe.inSync();
            fileStates.put(fe.getClientPath(), new P4File(fe));
         }
         log(fents.size() + " files (" + fileStates.size() + " total)");
      } finally {
         wc.hide();
      }
      SCMSystem.getEnvironment().getOverlays().updateOverlays();
   }

   final String getViewString(Client c) throws SCMException {
      try {
         final File root = new File(c.getRoot());
         final String absolute = root.getAbsolutePath();
         final String canonical = root.getCanonicalPath();
         if (!absolute.equals(canonical)) {
            final String msg = "The client root is not in canonical form. {0}"
               + "Please edit client and replace [{1}] with [{2}]";
            throw new SCMException(msg, new Object[] {
               System.getProperty("line.separator", "\n"), absolute, canonical,
            });
         }
         final String view = canonical + File.separator + "...";
         return view;
      } catch (IOException e) {
         throw new SCMException("I/O failure", e);
      }
   }


   /**
    * Determines if this AddIn is enabled.
    *
    * @return true
    */
   public boolean isSourceControlEnabled() {
      return true;
   }

   /**
    * Checks client configuration.
    *
    * @exception SCMException if client is not usable
    */
   public void detectClient() throws SCMException {
   }

   /**
    * Initializes this SCMClient.  This method is called whenever perforce
    * is available, even if it disabled, so no significant work is done here.
    *
    * @exception SCMException if an error occurs
    */
   public void prepareClient() throws SCMException {
      log(getSCMName() + " addin version " + version());
      try {
         final ConnectionManager cm = ConnectionManager.getInstance();
         final String[] names = // client names
            cm.getConnectionNamesForClass(P4ConnectionType.CLASS);

         if (names.length == 0) {
            final Client dc = P4ConnectionFactory.defaultClient();
            log("No connections defined.  Using default " + dc.getName());
         }
         for (int i = 0; i < names.length; i++ ) {
            log(names[i] + " connection ready");
         }
      } catch (CMException e) {
         final String msg = "Connections Problem";
         throw new SCMException(msg, e);
      } catch (IOException e) {
         final String msg = "Connections Problem";
         throw new SCMException(msg, e);
      }
   }

   /**
    * Tears down this addin and releases its resources.
    *
    * @exception SCMException if an unrecoverable error occurs
    */
   public void finishClient() throws SCMException {
      if (scmLog != null) {
         scmLog.close();
         scmLog = null;
      }
      scmOperations = null;
      fileStates.clear();
      try {
         final ConnectionManager cm = ConnectionManager.getInstance();
         final String[] names = // client names
            cm.getConnectionNamesForClass(P4ConnectionType.CLASS);

         for (int i = 0; i < names.length; i++ ) {
            cm.getConnection(names[i]).closeConnection();
         }
      } catch (CMException e) {
         final String msg = "May be open Connections";
         throw new SCMException(msg, e);
      } catch (IOException e) {
         final String msg = "May be open Connections";
         throw new SCMException(msg, e);
      }
   }

   /**
    * Finalization support.
    *
    * @exception Throwable on error
    */
   public void finalize() throws Throwable {
      finishClient();
      super.finalize();
   }


   /**
    * Registers with the source control addin framework.
    * An SCMClient must ensure that the addin is correctly
    * initialized before general operation.
    */
   public void initialize() {
      final SCMRegistry reg = SCMSystem.getRegistry();
      reg.register(this);
      final ConnectionManager cm = ConnectionManager.getInstance();
      cm.registerConnectionType(new P4ConnectionType());
   }

   /** An SCMClient must veto inappropriately timed shutdown requests. */
   public void shutdown() {
      if (!Ide.isQuitting()) {
         final Exception e = new ChangeVetoException("Not Quiting");
         SCMAssert.fail(e);
      }
   }

   /**
    * Called when the IDE is shutdown to allow the addin to alert the
    * user if it has some unfinished business, such a running process or
    * unsaved data. Use this method to give the user an opportunity to cancel
    * the shutdown process, and return the user's choice to the Addin Manager.
    *
    * <p> An SCMClient must veto inappropriately timed shutdown requests.
    *
    * @return <code>true</code> if shutdown is acceptable
    */
   public boolean canShutdown() {
      return Ide.isQuitting();
   }

   /**
    * Reports the version of this AddIn to the IDE.
    *
    * @return RELEASE.REVISION
    */
   public float version() {
      try {
         final int dollar = REVISION.lastIndexOf('$');
         final String revision = REVISION.substring(12, dollar).trim();
         final String version = RELEASE + "." + revision;
         return Float.parseFloat(version);
      } catch (NumberFormatException e) {
         return 0.0f;
      }
   }

   /**
    * Reports the version of the IDE loaded with this AddIn.
    *
    * @return 5.0 or higher
    */
   public float ideVersion() {
      return Ide.IDE_VERSION;
   }

}
