package com.perforce.jdev.cm;
import com.perforce.api.Client;

import java.io.IOException;

import java.net.InetAddress;
import java.net.UnknownHostException;

import java.util.WeakHashMap;
import java.util.Map;

import oracle.ide.scm.util.SCMEnvironmentVars;
import oracle.ide.scm.util.SCMOperatingSystem;

import oracle.jdeveloper.cm.CMException;
import oracle.jdeveloper.cm.ConnectInfoPrompter;
import oracle.jdeveloper.cm.ConnectionDescriptor;
import oracle.jdeveloper.cm.ConnectionFactory;
import oracle.jdeveloper.cm.ConnectionManager;
import oracle.jdeveloper.cm.ConnectionWrapper;


/**
 * Factory class for P4ConnectionWrapper.
 *
 * @author Paul Krause
 * @version $Revision: #2 $ $Date: 2002/10/08 $
 */
public class P4ConnectionFactory implements ConnectionFactory {

   /** Previously created connections. */
   private final Map wrapperCache = new WeakHashMap();

   /**
    * Entry point for <code>ConnectionManager</code> to
    * construct a <code>P4ConnectionWrapper</code>.
    *
    * @param cd a <code>ConnectionDescriptor</code> value
    * @param cip a <code>ConnectInfoPrompter</code> value
    * @param cm a <code>ConnectionManager</code> value
    * @return a <code>ConnectionWrapper</code> value
    * @exception CMException if an error occurs
    */
   public ConnectionWrapper createConnection(ConnectionDescriptor cd,
                                             ConnectInfoPrompter cip,
                                             ConnectionManager cm)
      throws CMException
   {
      final String name = cd.getConnectionName();
      final Object v = wrapperCache.get(name);
      if (v != null) return (ConnectionWrapper) v;
      final ConnectionWrapper w = new P4ConnectionWrapper(cd);
      wrapperCache.put(name, w);
      return w;
   }



   /**
    * Gets the name of the local host.
    *
    * @return The unqualified hostname
    */
   private final static String localHostName() {
      try {
         final String fullname = InetAddress.getLocalHost().getHostName();
         final int dot = fullname.indexOf('.');
         if (dot == -1) return fullname;
         return fullname.substring(0, dot);
      } catch (UnknownHostException e) {
         return "mycomputer";
      }
   }

   static ConnectionDescriptor defaultConnectionDescriptor() {
      final String executable = SCMOperatingSystem.isUnix() ?
         "/usr/local/bin/p4" : "C:\\Program Files\\Perforce\\p4.exe";

      final Map env = SCMEnvironmentVars.getInstance().getVariables();

      final String port = env.containsKey("P4PORT") ?
         (String) env.get("P4PORT") : "perforce:1666";
      final String user = env.containsKey("P4USER") ?
         (String) env.get("P4USER") : System.getProperty("user.name");
      final String localhost = env.containsKey("P4HOST") ?
         (String) env.get("P4HOST") : localHostName();
      final String client = env.containsKey("P4CLIENT") ? 
         (String) env.get("P4CLIENT") :
         (user + "-" + localhost + "-full").toLowerCase();

      final ConnectionDescriptor cd = new ConnectionDescriptor();

      cd.setProperty("p4.user", user);
      cd.setProperty("p4.client", client);
      cd.setProperty("p4.port", port);
      cd.setProperty("p4.executable", executable);

      final int colon = port.indexOf(':');
      final String hostname = (colon != -1) ?
         port.substring(0, colon) : (port.length() != 0) ? port : "perforce";
      final String hostport = (colon != -1) ?
         port.substring(colon) : "1666";

      cd.setProperty(ConnectionDescriptor.CONNECTION_NAME, client);
      cd.setProperty(ConnectionDescriptor.CONNECTION_TYPE, P4ConnectionType.TYPE);
      cd.setProperty(ConnectionDescriptor.HOSTNAME, hostname);
      cd.setProperty(ConnectionDescriptor.USERNAME, user);
      cd.setProperty(ConnectionDescriptor.PORT, hostport);

      return cd;
   }

   private static Client defaultClient;

   public synchronized static Client defaultClient()
      throws CMException, IOException
   {
      if (defaultClient != null) return defaultClient;
      final P4ConnectionWrapper w = defaultWrapper();
      defaultClient = (Client) w.getPresentation(P4ConnectionWrapper.CLIENT);
      return defaultClient;
   }

   static P4ConnectionWrapper defaultWrapper()
      throws CMException, IOException
   {
      final ConnectionDescriptor cd = defaultConnectionDescriptor();
      final String name = cd.getConnectionName();
      final ConnectionManager cm = ConnectionManager.getInstance();
      final ConnectionWrapper w = cm.getConnection(name);
      if (w != null) return (P4ConnectionWrapper) w;
      cm.putConnectionDescriptor(name, cd);
      return (P4ConnectionWrapper) cm.getConnection(name);
   }


}
